// BSP : STM32 boards groups by STMicroelectronics 2.7.1
// Borad select : STM Boards groups / LoRa boars
// Board part number : LoRa-E5 mini (Board to be selected from Tools submenu 'Board part number)

// Shutdown Current　
//    LoRa-E5 mini           70uA　 3.6V is applied to the BAT+ pin (AP2112 Iq=typ.55uA, CP2102 Iregin=typ.80uA)
//    Grove LoRa-E5(no LDO)  0.7uA  3.3V is applied to the VCC pin

// 2024/02/04

#include <Arduino.h>
#include <RadioLib.h>           // 6.4.1  https://github.com/jgromes/RadioLib
#include <STM32LowPower.h>      // 1.2.5  https://github.com/stm32duino/STM32LowPower
#include <STM32RTC.h>           // 1.4.0  https://github.com/stm32duino/STM32RTC

//#define LED PB5       // LoRa-E5 mini
#define LED PA9       // Grove LoRa-E5

// RM0461 Reference manual Rev 8  37.1.4 IEEE 64-bit unique device ID register (UID64)
#define TX_UID   uint32_t(*((uint32_t*)0x1FFF7580))    // use uid64 dor Slave device UID

// \Arduino15\packages\STMicroelectronics\hardware\stm32\2.7.1
// \variants\STM32WLxx\WL54JCI_WL55JCI_WLE4J(8-B-C)I_WLE5J(8-B-C)I\variant_LORA_E5_MINI.h : 173
STM32WLx radio = new STM32WLx_Module();
static const uint32_t rfswitch_pins[] =
                         {PA4,  PA5,  RADIOLIB_NC};
static const Module::RfSwitchMode_t rfswitch_table[] = {
  {STM32WLx::MODE_IDLE,  {LOW,  LOW}},
  {STM32WLx::MODE_RX,    {HIGH, LOW}},
  {STM32WLx::MODE_TX_HP, {LOW, HIGH}},  // for LoRa-E5 mini
//  {STM32WLx::MODE_TX_LP, {HIGH, HIGH}},   // for LoRa-E5-LE mini
  END_OF_MODE_TABLE,
};

#define dataNum   12                // data bytes
union unionData {                   // for data type conversion
  uint32_t  dataBuff32[dataNum/4];
  float     dataBuff32F[dataNum/4];
  uint16_t  dataBuff16[dataNum/2];
  uint8_t   dataBuff8[dataNum];
};
union unionData ud;

int bps;                // data rate [bps]
uint16_t sec;           // mS/1000 data [sec]
char printBuff[4];      // for sprintf()
String  txdata = "";    // transmission data packet string

// *******************************************************************************************************
void setup() {
  // Pin initialization
  pinMode(LED, OUTPUT);         // built-in LED
  digitalWrite(LED, HIGH);      // LED off

  // Radio initialization
  radio.setRfSwitchTable(rfswitch_pins, rfswitch_table);

  // int16_t begin(float freq = 434.0, float bw = 125.0, uint8_t sf = 9, uint8_t cr = 7, \
  // uint8_t syncWord = RADIOLIB_SX126X_SYNC_WORD_PRIVATE, int8_t power = 10, \
  // uint16_t preambleLength = 8, float tcxoVoltage = 1.6, bool useRegulatorLDO = false);
  int state = radio.begin(922.6, 125.0, 7, 5, 0x12, 13, 8, 1.7, 0);

  if (state == RADIOLIB_ERR_NONE) {
      digitalWrite(LED, LOW);
      delay(100);
      digitalWrite(LED, HIGH);
      delay(100);      
  }
  else {
    Serial.print("failed, code ");
    Serial.println(state);
    while(true) {
      digitalWrite(LED, LOW);
      delay(100);
      digitalWrite(LED, HIGH);
      delay(100);      
    }
  }

  // Configure low power
  LowPower.begin();

}

// ***********************************************************************************************************
void loop() 
{
  // Slave transmits data
  ud.dataBuff32[0] = TX_UID;                      // TX Device UID
  ud.dataBuff32[1] = millis();                    // transmission time
  ud.dataBuff32[2] = micros();

  digitalWrite(LED, LOW);
  int state = radio.transmit(ud.dataBuff8, dataNum);
  digitalWrite(LED, HIGH);

//  delay(1000);

  // radio Sleep mode
  radio.sleep();
//  delay(1000);

  // Shutdown
  LowPower.shutdown(10000);
}
